@@ -6,6 +6,9 @@
 */
 
 #include "karchivetest.h"
+
+#include "ossfuzz/karchive_fuzzer_common.h"
+
 #include <k7zip.h>
 #include <kar.h>
 #include <krcc.h>
@@ -39,9 +42,9 @@ QTEST_MAIN(KArchiveTest)
 
 static const int SIZE1 = 100;
 
-/**
+/*!
  * Writes test fileset specified archive
- * @param archive archive
+ * \a archive archive
  */
 static void writeTestFilesToArchive(KArchive *archive)
 {
@@ -160,9 +163,9 @@ static QStringList recursiveListEntries(const KArchiveDirectory *dir, const QStr
     return ret;
 }
 
-/**
+/*!
  * Verifies contents of specified archive against test fileset
- * @param archive archive
+ * \a archive archive
  */
 static void testFileData(KArchive *archive)
 {
@@ -329,7 +332,7 @@ static void testCopyTo(KArchive *archive)
     QVERIFY(QFileInfo(dirName + "executableAll").permissions() & (QFileDevice::ExeOwner | QFileDevice::ExeGroup | QFileDevice::ExeOther));
 }
 
-/**
+/*!
  * Prepares dataset for archive filter tests
  */
 void KArchiveTest::setupData()
@@ -346,6 +349,8 @@ void KArchiveTest::setupData()
 #if HAVE_XZ_SUPPORT
     QTest::newRow(".tar.lzma") << "karchivetest.tar.lzma"
                                << "application/x-lzma";
+    QTest::newRow(".tar.lz") << "karchivetest.tar.lz"
+                             << "application/x-lzip";
     QTest::newRow(".tar.xz") << "karchivetest.tar.xz"
                              << "application/x-xz";
 #endif
@@ -355,8 +360,8 @@ void KArchiveTest::setupData()
 #endif
 }
 
-/**
- * @see QTest::initTestCase()
+/*!
+ * \sa QTest::initTestCase()
  */
 void KArchiveTest::initTestCase()
 {
@@ -402,7 +407,7 @@ void KArchiveTest::testCreateTar_data()
     QTest::newRow(".tar") << "karchivetest.tar";
 }
 
-/**
+/*!
  * @dataProvider testCreateTar_data
  */
 void KArchiveTest::testCreateTar()
@@ -431,7 +436,7 @@ void KArchiveTest::testCreateTar()
     // QFile::remove(fileName);
 }
 
-/**
+/*!
  * @dataProvider setupData
  */
 void KArchiveTest::testCreateTarXXX()
@@ -473,7 +478,7 @@ void KArchiveTest::testCreateTarXXX()
         QVERIFY(dt.secsTo(expectedDateTime) <= 2);                                                                                                             \
     }
 
-/**
+/*!
  * @dataProvider setupData
  */
 void KArchiveTest::testReadTar() // testCreateTarGz must have been run first.
@@ -545,7 +550,7 @@ void KArchiveTest::testReadTar() // testCreateTarGz must have been run first.
     }
 }
 
-/**
+/*!
  * This tests the decompression using kfilterdev, basically.
  * To debug KTarPrivate::fillTempFile().
  *
@@ -577,7 +582,7 @@ void KArchiveTest::testUncompress()
     QVERIFY(totalSize > 26000); // 27648 here when using gunzip
 }
 
-/**
+/*!
  * @dataProvider setupData
  */
 void KArchiveTest::testTarFileData()
@@ -593,7 +598,7 @@ void KArchiveTest::testTarFileData()
     QVERIFY(tar.close());
 }
 
-/**
+/*!
  * @dataProvider setupData
  */
 void KArchiveTest::testTarCopyTo()
@@ -609,7 +614,7 @@ void KArchiveTest::testTarCopyTo()
     QVERIFY(tar.close());
 }
 
-/**
+/*!
  * @dataProvider setupData
  */
 void KArchiveTest::testTarReadWrite()
@@ -617,13 +622,15 @@ void KArchiveTest::testTarReadWrite()
     QFETCH(QString, fileName);
 
     // testCreateTar must have been run first.
-    KTar tar(fileName);
-    QVERIFY(tar.open(QIODevice::ReadWrite));
+    {
+        KTar tar(fileName);
+        QVERIFY(tar.open(QIODevice::ReadWrite));
 
-    testReadWrite(&tar);
-    testFileData(&tar);
+        testReadWrite(&tar);
+        testFileData(&tar);
 
-    QVERIFY(tar.close());
+        QVERIFY(tar.close());
+    }
 
     // Reopen it and check it
     {
@@ -647,7 +654,7 @@ void KArchiveTest::testTarMaxLength_data()
     QTest::newRow("maxlength.tar.gz") << "karchivetest-maxlength.tar.gz";
 }
 
-/**
+/*!
  * @dataProvider testTarMaxLength_data
  */
 void KArchiveTest::testTarMaxLength()
@@ -894,6 +901,15 @@ void KArchiveTest::testTarDirectoryTwice() // bug 206994
     QCOMPARE(listing.count(), 3);
 }
 
+void KArchiveTest::testTarGzHugeMemoryUsage()
+{
+    const QString filePath = QFINDTESTDATA("data/ossfuzz_testcase_6581632288227328.tar.gz");
+    QVERIFY(!filePath.isEmpty());
+
+    KTar tar(filePath);
+    tar.open(QIODevice::ReadOnly);
+}
+
 void KArchiveTest::testTarIgnoreRelativePathOutsideArchive()
 {
 #if HAVE_BZIP2_SUPPORT
@@ -919,6 +935,36 @@ void KArchiveTest::testTarIgnoreRelativePathOutsideArchive()
     QSKIP("Test data is in bz2 format and karchive is built without bzip2 format");
 #endif
 }
+
+void KArchiveTest::testTarDeepDirHierarchy()
+{
+#if HAVE_XZ_SUPPORT
+    const QString fileName = QFINDTESTDATA("data/tar_deep_hierarchy.tar.xz");
+    QVERIFY(!fileName.isEmpty());
+
+    KTar tar(fileName);
+    QVERIFY(tar.open(QIODevice::ReadOnly));
+    traverseArchive(tar.directory());
+#else
+    QSKIP("Test data is in xz format and karchive is built without xz format");
+#endif
+}
+
+void KArchiveTest::benchmarkTarDeepDirHierarchy()
+{
+#if HAVE_XZ_SUPPORT
+    const QString fileName = QFINDTESTDATA("data/tar_deep_hierarchy.tar.xz");
+    QVERIFY(!fileName.isEmpty());
+
+    QBENCHMARK {
+        KTar tar(fileName);
+        QVERIFY(tar.open(QIODevice::ReadOnly));
+    }
+#else
+    QSKIP("Test data is in xz format and karchive is built without xz format");
+#endif
+}
+
 ///
 
 static const char s_zipFileName[] = "karchivetest.zip";
@@ -1216,7 +1262,7 @@ void KArchiveTest::testZipReadRedundantDataDescriptor_data()
     QTest::newRow("withSignature") << "data/redundantDataDescriptorsWithSignature.zip";
 }
 
-/**
+/*!
  * @dataProvider testZipReadRedundantDataDescriptor_data
  */
 void KArchiveTest::testZipReadRedundantDataDescriptor()
@@ -1276,6 +1322,31 @@ void KArchiveTest::testZipWithinZip()
     QCOMPARE(listing[1], QString::fromUtf8("mode=100644 path=test.epub type=file size=525154"));
 }
 
+void KArchiveTest::testZipPrependedData()
+{
+    const QString fileName = QFINDTESTDATA("data/zip64_datadescriptor.zip");
+    QVERIFY(!fileName.isEmpty());
+
+    QFile origFile(fileName);
+    QVERIFY2(origFile.open(QIODevice::ReadOnly), qPrintable(origFile.errorString()));
+    auto zipData = origFile.readAll();
+    QVERIFY(zipData.startsWith("PK"));
+
+    zipData.insert(0, QByteArrayLiteral("PrependedPK\x01\x02Garbage"));
+    QBuffer zipBuffer(&zipData);
+
+    KZip zip(&zipBuffer);
+    QVERIFY2(zip.open(QIODevice::ReadOnly), qPrintable(zip.errorString()));
+
+    QCOMPARE(zip.directory()->entries().size(), 1);
+    QCOMPARE(zip.directory()->entries(), QList{QStringLiteral("-")});
+
+    auto entry = zip.directory()->file(QStringLiteral("-"));
+    QVERIFY(entry);
+    QCOMPARE(entry->size(), 4);
+    QCOMPARE(entry->data(), "abcd");
+}
+
 void KArchiveTest::testZipUnusualButValid()
 {
     const QString fileName = QFINDTESTDATA("data/unusual_but_valid_364071.zip");
@@ -1330,6 +1401,230 @@ void KArchiveTest::testZip64()
     QCOMPARE(entry->position(), 8943);
 }
 
+void KArchiveTest::testZipReopenWithoutDoubleDeletion()
+{
+    // testCreateZip must have been run first.
+    KZip zip(s_zipFileName);
+
+    QVERIFY(zip.open(QIODevice::ReadOnly));
+    QVERIFY(zip.close());
+
+    QVERIFY(zip.open(QIODevice::WriteOnly));
+    QVERIFY(zip.close()); // should not crash
+}
+
+void KArchiveTest::testZip64NestedStored()
+{
+    const QString fileName = QFINDTESTDATA("data/dirpermissions.zip");
+    QVERIFY(!fileName.isEmpty());
+    QFile nested(fileName);
+    QVERIFY(nested.open(QIODevice::ReadOnly));
+    auto nestedData = nested.readAll();
+
+    QBuffer archive;
+
+    KZip writer(&archive);
+    QVERIFY(writer.open(QIODevice::WriteOnly));
+    // Write nested zip file
+    writer.setCompression(KZip::NoCompression);
+    QVERIFY(writer.prepareWriting(QStringLiteral("nested.zip"), {}, {}, nestedData.size()));
+    QVERIFY(writer.writeData(nestedData));
+    QVERIFY(writer.finishWriting(nestedData.size()));
+
+    QVERIFY(writer.close());
+
+    KZip reader(&archive);
+    QVERIFY(reader.open(QIODevice::ReadOnly));
+
+    QCOMPARE(reader.directory()->entries().size(), 1);
+    QCOMPARE(reader.directory()->entries(), QList{QStringLiteral("nested.zip")});
+
+    auto entry = reader.directory()->file(QStringLiteral("nested.zip"));
+    QVERIFY(entry);
+    QCOMPARE(entry->size(), 430);
+    QCOMPARE(entry->size(), nestedData.size());
+
+    QVERIFY(reader.close());
+}
+
+void KArchiveTest::testZip64NestedStoredStreamed()
+{
+    const QString fileName = QFINDTESTDATA("data/zip64_nested_stored_streamed.zip");
+    QVERIFY(!fileName.isEmpty());
+
+    KZip zip(fileName);
+    QVERIFY2(zip.open(QIODevice::ReadOnly), qPrintable(zip.errorString()));
+
+    QCOMPARE(zip.directory()->entries().size(), 2);
+
+    auto entry = zip.directory()->file(QStringLiteral("zip64_datadescriptor.zip"));
+    QVERIFY(entry);
+    QCOMPARE(entry->size(), 150);
+
+    entry = zip.directory()->file(QStringLiteral("zip64_end_of_central_directory.zip"));
+    QVERIFY(entry);
+    QCOMPARE(entry->size(), 200);
+
+    QVERIFY(zip.close());
+}
+
+void KArchiveTest::testZip64EndOfCentralDirectory()
+{
+    const QString fileName = QFINDTESTDATA("data/zip64_end_of_central_directory.zip");
+    QVERIFY(!fileName.isEmpty());
+
+    KZip zip(fileName);
+    QVERIFY2(zip.open(QIODevice::ReadOnly), qPrintable(zip.errorString()));
+
+    QCOMPARE(zip.directory()->entries().size(), 1);
+    QCOMPARE(zip.directory()->entries(), QList{QStringLiteral("-")});
+
+    auto entry = zip.directory()->file(QStringLiteral("-"));
+    QVERIFY(entry);
+    QCOMPARE(entry->size(), 4);
+
+    QVERIFY(zip.close());
+}
+
+void KArchiveTest::testZip64DataDescriptor()
+{
+    const QString fileName = QFINDTESTDATA("data/zip64_datadescriptor.zip");
+    QVERIFY(!fileName.isEmpty());
+
+    KZip zip(fileName);
+    QVERIFY2(zip.open(QIODevice::ReadOnly), qPrintable(zip.errorString()));
+
+    QCOMPARE(zip.directory()->entries().size(), 1);
+    QCOMPARE(zip.directory()->entries(), QList{QStringLiteral("-")});
+
+    auto entry = zip.directory()->file(QStringLiteral("-"));
+    QVERIFY(entry);
+    QCOMPARE(entry->size(), 4);
+    QCOMPARE(entry->data(), "abcd");
+
+    QVERIFY(zip.close());
+}
+
+void KArchiveTest::testZip64ExtraZip64Size()
+{
+    const QString fileName = QFINDTESTDATA("data/zip64_extra_zip64_size.zip.gz");
+    QVERIFY(!fileName.isEmpty());
+
+    KCompressionDevice gzfilter(fileName);
+    QVERIFY2(gzfilter.open(QIODevice::ReadOnly), qPrintable(gzfilter.errorString()));
+
+    // Get the actual test data, which is compressed so the repository does
+    // not have to store 4 MByte of binary data
+    auto zipData = gzfilter.read(5 * 1000 * 1000);
+    QVERIFY(zipData.startsWith("PK"));
+
+    QBuffer zipBuffer(&zipData);
+    KZip zip(&zipBuffer);
+    QVERIFY2(zip.open(QIODevice::ReadOnly), qPrintable(zip.errorString()));
+
+    QCOMPARE(zip.directory()->entries().size(), 1);
+    QCOMPARE(zip.directory()->entries(), QList{QStringLiteral("-")});
+
+    auto entry = zip.directory()->file(QStringLiteral("-"));
+    QVERIFY(entry);
+    QCOMPARE(entry->size(), 4404019208);
+
+    auto readDev = std::unique_ptr<QIODevice>(entry->createDevice());
+    auto head = readDev->read(8);
+    QCOMPARE(head, QByteArrayLiteral("abcd\0\0\0\0"));
+    readDev->seek(entry->size() - 8);
+    auto tail = readDev->read(8);
+    QCOMPARE(tail, QByteArrayLiteral("\0\0\0\0ABCD"));
+
+    QVERIFY(zip.close());
+}
+
+void KArchiveTest::testZip64ExtraZip64SizeFirst()
+{
+    const QString fileName = QFINDTESTDATA("data/zip64_extra_zip64_size_first.zip.gz");
+    QVERIFY(!fileName.isEmpty());
+
+    KCompressionDevice gzfilter(fileName);
+    QVERIFY2(gzfilter.open(QIODevice::ReadOnly), qPrintable(gzfilter.errorString()));
+
+    // Get the actual test data, which is compressed so the repository does
+    // not have to store 4 MByte of binary data
+    auto zipData = gzfilter.read(5 * 1000 * 1000);
+    QVERIFY(zipData.startsWith("PK"));
+
+    QBuffer zipBuffer(&zipData);
+    KZip zip(&zipBuffer);
+    QEXPECT_FAIL("", "Zip64 extra at front incorrectly parsed", Abort);
+    QVERIFY2(zip.open(QIODevice::ReadOnly), qPrintable(zip.errorString()));
+
+    QCOMPARE(zip.directory()->entries().size(), 2);
+    QCOMPARE(zip.directory()->entries(), (QList{QStringLiteral("4200M.bin"), QStringLiteral("4.bin")}));
+
+    auto entry = zip.directory()->file(QStringLiteral("4200M.bin"));
+    QVERIFY(entry);
+    QCOMPARE(entry->size(), 4404019208);
+
+    auto readDev = std::unique_ptr<QIODevice>(entry->createDevice());
+    auto head = readDev->read(8);
+    QCOMPARE(head, QByteArrayLiteral("abcd\0\0\0\0"));
+    readDev->seek(entry->size() - 8);
+    auto tail = readDev->read(8);
+    QCOMPARE(tail, QByteArrayLiteral("\0\0\0\0ABCD"));
+
+    QVERIFY(zip.close());
+}
+
+void KArchiveTest::testZip64ExtraZip64Offset()
+{
+    const QString fileName = QFINDTESTDATA("data/zip64_extra_zip64_localheader.oxps");
+    QVERIFY(!fileName.isEmpty());
+
+    KZip zip(fileName);
+    QVERIFY2(zip.open(QIODevice::ReadOnly), qPrintable(zip.errorString()));
+
+    QCOMPARE(zip.directory()->entries().size(), 6);
+
+    auto entry = zip.directory()->file(QStringLiteral("_rels/.rels"));
+    QVERIFY(entry);
+    QCOMPARE(entry->size(), 576);
+
+    const auto data = entry->data();
+    QCOMPARE(data.sliced(0, 6), "<?xml ");
+
+    QVERIFY(zip.close());
+}
+
+void KArchiveTest::testZipOssFuzzIssue433303801()
+{
+    const QString fileName = QFINDTESTDATA("data/ossfuzz_issue_433303801.zip");
+    QVERIFY(!fileName.isEmpty());
+
+    KZip zip(fileName);
+    QVERIFY(zip.open(QIODevice::ReadOnly));
+    traverseArchive(zip.directory());
+}
+
+void KArchiveTest::testZipDeepDirHierarchy()
+{
+    const QString fileName = QFINDTESTDATA("data/zip_deep_hierarchy.zip");
+    QVERIFY(!fileName.isEmpty());
+
+    KZip zip(fileName);
+    QVERIFY(zip.open(QIODevice::ReadOnly));
+    traverseArchive(zip.directory());
+}
+
+void KArchiveTest::benchmarkZipDeepDirHierarchy()
+{
+    const QString fileName = QFINDTESTDATA("data/zip_deep_hierarchy.zip");
+    QVERIFY(!fileName.isEmpty());
+
+    QBENCHMARK {
+        KZip zip(fileName);
+        QVERIFY(zip.open(QIODevice::ReadOnly));
+    }
+}
+
 void KArchiveTest::testRcc()
 {
     const QString rccFile = QFINDTESTDATA("data/runtime_resource.rcc"); // was copied from qtbase/tests/auto/corelib/io/qresourceengine
@@ -1382,8 +1677,8 @@ void KArchiveTest::testAr()
     QVERIFY(ar.close());
 }
 
-/**
- * @see QTest::cleanupTestCase()
+/*!
+ * \sa QTest::cleanupTestCase()
  */
 void KArchiveTest::cleanupTestCase()
 {
@@ -1399,7 +1694,7 @@ void KArchiveTest::cleanupTestCase()
 
 #if HAVE_XZ_SUPPORT
 
-/**
+/*
  * Prepares dataset for 7zip tests
  */
 void KArchiveTest::setup7ZipData()
@@ -1408,7 +1703,7 @@ void KArchiveTest::setup7ZipData()
     QTest::newRow(".7z") << "karchivetest.7z";
 }
 
-/**
+/*
  * @dataProvider testCreate7Zip_data
  */
 void KArchiveTest::testCreate7Zip()
@@ -1431,7 +1726,7 @@ void KArchiveTest::testCreate7Zip()
     }
 }
 
-/**
+/*!
  * @dataProvider setupData
  */
 void KArchiveTest::testRead7Zip() // testCreate7Zip must have been run first.
@@ -1480,7 +1775,7 @@ void KArchiveTest::testRead7Zip() // testCreate7Zip must have been run first.
     }
 }
 
-/**
+/*!
  * @dataProvider setupData
  */
 void KArchiveTest::test7ZipFileData()
@@ -1496,7 +1791,7 @@ void KArchiveTest::test7ZipFileData()
     QVERIFY(k7zip.close());
 }
 
-/**
+/*!
  * @dataProvider setupData
  */
 void KArchiveTest::test7ZipCopyTo()
@@ -1512,7 +1807,7 @@ void KArchiveTest::test7ZipCopyTo()
     QVERIFY(k7zip.close());
 }
 
-/**
+/*!
  * @dataProvider setupData
  */
 void KArchiveTest::test7ZipReadWrite()
@@ -1520,13 +1815,15 @@ void KArchiveTest::test7ZipReadWrite()
     QFETCH(QString, fileName);
 
     // testCreate7zip must have been run first.
-    K7Zip k7zip(fileName);
-    QVERIFY(k7zip.open(QIODevice::ReadWrite));
+    {
+        K7Zip k7zip(fileName);
+        QVERIFY(k7zip.open(QIODevice::ReadWrite));
 
-    testReadWrite(&k7zip);
-    testFileData(&k7zip);
+        testReadWrite(&k7zip);
+        testFileData(&k7zip);
 
-    QVERIFY(k7zip.close());
+        QVERIFY(k7zip.close());
+    }
 
     // Reopen it and check it
     {
@@ -1544,7 +1841,7 @@ void KArchiveTest::test7ZipReadWrite()
     QFile::remove(fileName);
 }
 
-/**
+/*!
  * @dataProvider test7ZipMaxLength_data
  */
 void KArchiveTest::test7ZipMaxLength()
@@ -1632,6 +1929,151 @@ void KArchiveTest::test7ZipMultipleNamelessFiles()
 
     QVERIFY(k7zip.close());
 }
+
+// This test was added to verify K7ZipPrivate::readNumber() boundary check.
+// See: https://invent.kde.org/frameworks/karchive/-/merge_requests/105
+void KArchiveTest::test7ZipReadNumber()
+{
+    const QString fileName = QFINDTESTDATA("data/readnumber_check.7z");
+    QVERIFY(!fileName.isEmpty());
+
+    K7Zip k7zip(fileName);
+    QVERIFY2(k7zip.open(QIODevice::ReadOnly), "data/readnumber_check.7z");
+
+    const KArchiveDirectory *dir = k7zip.directory();
+    QVERIFY(dir != nullptr);
+    const QStringList listing = recursiveListEntries(dir, QLatin1String(""), 0);
+
+    QCOMPARE(listing.count(), 1);
+    QCOMPARE(listing[0], QString("mode=644 path=file.txt type=file size=5"));
+
+    QVERIFY(k7zip.close());
+}
+
+void KArchiveTest::test7ZipFileNameEndsInSlash()
+{
+    const QString fileName = QFINDTESTDATA("data/filename_ends_in_slash.7z");
+    QVERIFY(!fileName.isEmpty());
+
+    K7Zip k7zip(fileName);
+    QVERIFY(!k7zip.open(QIODevice::ReadOnly));
+}
+
+void KArchiveTest::test7ZipOssFuzzIssues_data()
+{
+    QTest::addColumn<QString>("fileName");
+    QTest::newRow("issue_440829292") << "data/ossfuzz_issue_440829292.7z";
+    QTest::newRow("issue_441906077") << "data/ossfuzz_issue_441906077.7z";
+    QTest::newRow("testcase_4566647131406336") << "data/ossfuzz_testcase_4566647131406336.7z";
+    QTest::newRow("testcase_4784793343819776") << "data/ossfuzz_testcase_4784793343819776.7z";
+    QTest::newRow("testcase_5082404562993152") << "data/ossfuzz_testcase_5082404562993152.7z";
+    QTest::newRow("testcase_5132547098214400") << "data/ossfuzz_testcase_5132547098214400.7z";
+    QTest::newRow("testcase_5141245598171136") << "data/ossfuzz_testcase_5141245598171136.7z";
+    QTest::newRow("testcase_5560695602348032") << "data/ossfuzz_testcase_5560695602348032.7z";
+    QTest::newRow("testcase_5581009572921344") << "data/ossfuzz_testcase_5581009572921344.7z";
+    QTest::newRow("testcase_5821682553257984") << "data/ossfuzz_testcase_5821682553257984.7z";
+    QTest::newRow("testcase_5991129817612288") << "data/ossfuzz_testcase_5991129817612288.7z";
+    QTest::newRow("testcase_6077171694370816") << "data/ossfuzz_testcase_6077171694370816.7z";
+    QTest::newRow("testcase_6096742417498112") << "data/ossfuzz_testcase_6096742417498112.7z";
+    QTest::newRow("testcase_6213340184772608") << "data/ossfuzz_testcase_6213340184772608.7z";
+    QTest::newRow("testcase_6248361801089024") << "data/ossfuzz_testcase_6248361801089024.7z";
+    QTest::newRow("testcase_6366650283917312") << "data/ossfuzz_testcase_6366650283917312.7z";
+    QTest::newRow("testcase_6532014901886976") << "data/ossfuzz_testcase_6532014901886976.7z";
+    QTest::newRow("testcase_6593198198947840") << "data/ossfuzz_testcase_6593198198947840.7z";
+    QTest::newRow("testcase_6653406484955136") << "data/ossfuzz_testcase_6653406484955136.7z";
+}
+
+void KArchiveTest::test7ZipOssFuzzIssues()
+{
+    QFETCH(QString, fileName);
+
+    const QString filePath = QFINDTESTDATA(fileName);
+    QVERIFY(!filePath.isEmpty());
+
+    K7Zip k7zip(filePath);
+    if (k7zip.open(QIODevice::ReadOnly)) {
+        k7zip.close();
+    }
+}
+
+#if HAVE_OPENSSL_SUPPORT
+void KArchiveTest::test7ZipPasswordProtected()
+{
+    const QString fileName = QFINDTESTDATA("data/password_protected.7z");
+    QVERIFY(!fileName.isEmpty());
+
+    K7Zip k7zip(fileName);
+    QVERIFY(!k7zip.open(QIODevice::ReadOnly)); // Should fail without a password
+
+    QVERIFY(k7zip.passwordNeeded());
+    k7zip.setPassword("youshallnotpass");
+
+    QVERIFY2(k7zip.open(QIODevice::ReadOnly), "data/password_protected.7z");
+
+    const KArchiveDirectory *dir = k7zip.directory();
+    QVERIFY(dir != nullptr);
+
+    const KArchiveEntry *entry = dir->entry("file.txt");
+    QVERIFY(entry != nullptr);
+
+    QVERIFY(entry->name() == "file.txt");
+    QVERIFY(entry->isFile());
+
+    const QByteArray fileData = QByteArray("Hello from an encrypted file in a 7z archive!\n");
+
+    const KArchiveFile *file = static_cast<const KArchiveFile *>(entry);
+    QCOMPARE(file->data(), fileData);
+
+    QVERIFY(k7zip.close());
+}
+#endif
+
+void KArchiveTest::test7ZipReadCoder_data()
+{
+    QTest::addColumn<QString>("fileName");
+
+    QTest::newRow("copy") << "data/7z_coder_test_copy.7z";
+    QTest::newRow("lzma") << "data/7z_coder_test_lzma.7z";
+    QTest::newRow("lzma2") << "data/7z_coder_test_lzma2.7z";
+    QTest::newRow("deflate") << "data/7z_coder_test_deflate.7z";
+    QTest::newRow("bcj") << "data/7z_coder_test_bcj.7z";
+    QTest::newRow("bcj2") << "data/7z_coder_test_bcj2.7z";
+    QTest::newRow("bzip2") << "data/7z_coder_test_bzip2.7z";
+    QTest::newRow("zstd") << "data/7z_coder_test_zstd.7z";
+}
+
+/*!
+ * @dataProvider test7ZipReadCoder_data
+ */
+void KArchiveTest::test7ZipReadCoder()
+{
+    QFETCH(QString, fileName);
+
+    const QString filePath = QFINDTESTDATA(fileName);
+    QVERIFY(!filePath.isEmpty());
+
+    K7Zip k7zip(filePath);
+    QVERIFY(k7zip.open(QIODevice::ReadOnly));
+
+    const KArchiveDirectory *dir = k7zip.directory();
+    QVERIFY(dir != nullptr);
+
+    const KArchiveEntry *entry = dir->entry("hello.txt");
+    QVERIFY(entry != nullptr);
+    QVERIFY(entry->isFile());
+
+    const QByteArray fileData = QByteArray("Hello, World!\n");
+
+    const KArchiveFile *file = static_cast<const KArchiveFile *>(entry);
+
+    if (fileName.contains("bcj2")) {
+        QEXPECT_FAIL("", "bcj2 decoding fails, see https://bugs.kde.org/show_bug.cgi?id=509201", Continue);
+    }
+    QCOMPARE(file->data(), fileData);
+
+    QVERIFY(k7zip.close());
+}
+
 #endif
 
 #include "moc_karchivetest.cpp"
